/*
** Copyright (C) 2001,2002 Sacha Faust <sacha@severus.org>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
/*
 * Metis.java
 *
 * Created on November 16, 2001, 11:21 PM
 */

package org.ideahamster.metis;

import faust.sacha.web.bot.spider.*;
import faust.sacha.web.data.*;
import faust.sacha.web.util.*;
import org.ideahamster.metis.stat.*;
import org.ideahamster.metis.report.*;
import org.ideahamster.metis.util.*;
import gnu.getopt.Getopt;
import java.net.MalformedURLException;

/**
 *
 * @author Sacha Faust
 * @version 1.0
 */
public class Metis {

    /**
     */
    private Site m_site;
    private SiteStats m_siteStats;
    
//------------------------------------------------------------------------------
    public Metis() throws MalformedURLException {
        WebGlobal.setUserAgent( MetisGlobal.METIS_USER_AGENT );
        //WebGlobal.enableSSLDynamically();
        m_site = new Site( MetisOpt.getrootURL() );

    }
//------------------------------------------------------------------------------    
    public void run(){
        collectData();
        processData();
        reportData();
        
        System.out.println("Metis done for site : " + m_site.getDomain());
    }
//------------------------------------------------------------------------------    
    private void collectData(){
        
        Webspider bot = new Webspider(m_site);
        bot.run();
        
        /*
         *  Doing a bit of cleanup
         */
        System.gc();
        System.runFinalization();
    }
//------------------------------------------------------------------------------
    private void processData(){
       getStats(); 
    }
//------------------------------------------------------------------------------
    private void getStats(){
        m_siteStats = new SiteStats(m_site); 
    }    
//------------------------------------------------------------------------------
    private void reportData(){
        MetisReport report = null;
        
        switch( MetisOpt.getReportType() ){
            case MetisGlobal.REPORT_TYPE_ASCII:
                report = (MetisReport)new MetisReportASCII( m_site, m_siteStats, m_site.getDomain() );
                break;
            case MetisGlobal.REPORT_TYPE_HTML:
                report = (MetisReport)new MetisReportHTML( m_site, m_siteStats );
                break;
            default:
                System.err.println("Unsupported report type. Using general ASCII report\n");
                report = (MetisReport)new MetisReportASCII( m_site, m_siteStats, m_site.getDomain() );
                break;
        }
        
        report.generateReport();
    }
//------------------------------------------------------------------------------    
//------------------------------------------------------------------------------
    public static void main( String args[] ){
        String site = "";
        Metis metis = null;
        MetisOpt options = new MetisOpt();
        Getopt opt = new Getopt("metis", args, "DRSRr:U:P:T:F:h:");
        int c;
        String argument = "";
        
        if( args.length < 1 )
            options.usage();
        
        while( (c = opt.getopt()) != -1 ){
            argument = "";
            switch( c ){
                case 'D':
                    options.setDebug(true);
                    break;
                case 'R':
                    // not implemented yet
                    break;
                case 'S':
                    // not implemented yet
                    break;
                case 'r':
                    argument = opt.getOptarg();
                    if( argument == null ){
                        System.err.println("You need to specify a request method!");
                        options.usage();
                    }
                    try{
                        options.setRequestMethod( Integer.parseInt(argument) );
                    }
                    catch( NumberFormatException requestNumEx ){
                        System.err.println(requestNumEx);
                        System.exit(1);
                    }
                    break;
                case 'U':
                    argument = opt.getOptarg();
                    if( argument == null ){
                        System.err.println("You need to specify a username!");
                        options.usage();
                    }
                    options.setRequestUserName(argument);
                    break;
                case 'P':
                    argument = opt.getOptarg();
                    if( argument == null ){
                        System.err.println("You need to specify a password!");
                        options.usage();
                    }
                    options.setRequestPassword(argument);
                    break;
                case 'T':
                    argument = opt.getOptarg();
                    if( argument == null ){
                        System.err.println("You need to specify a report type!");
                        options.usage();
                    }
                    try{
                        options.setReportType( Integer.parseInt(argument) );
                    }
                    catch( NumberFormatException reportNumEx ){
                        System.err.println(reportNumEx);
                        System.exit(1);
                    }
                    break;
                case 'F':
                    argument = opt.getOptarg();
                    if( argument == null ){
                        System.err.println("You need to specify a file name!");
                        options.usage();
                    }
                    options.setConfigFileName(argument);
                    break;
                case 'h':
                    argument = opt.getOptarg();
                    if( argument == null ){
                        System.err.println("You need to specify a site!");
                        options.usage();
                    }
                    options.setrootURL(argument);
                    break;
                default:
                    System.err.println("Unknown options : " + (char)c);
                    options.usage();
                    break;
            }
        }
        
        //validate the options
        if( !options.validateOptions() ){
            options.usage();
        }
        
        try{
            metis = new Metis();
        }
        catch( MalformedURLException urlEx ){
            System.err.println("Metis::main() : " + urlEx );
            System.exit(0);
        }
        
        metis.run();
        
        System.exit(0);
    }
}
