/*
** Copyright (C) 2001,2002 Sacha Faust <sacha@severus.org>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/*
 * Webspider.java
 *
 * Created on November 11, 2001, 2:26 AM
 */

package faust.sacha.web.bot.spider;

/**
 *
 * @author Sacha Faust
 * @version 1.0
 */
import org.apache.commons.httpclient.*;
import org.apache.commons.httpclient.methods.*;
import faust.sacha.web.data.*;
import faust.sacha.web.util.*;
import javax.swing.text.*;
import javax.swing.text.html.*;
import javax.swing.text.html.parser.*;
import java.util.*;
import java.io.*;
import java.net.MalformedURLException;

public class Webspider {

    /**
     */
    private Site m_site;
    private int m_requestMethodType;
    
    private Webspider() {
        m_requestMethodType = WebGlobal.REQUEST_METHOD_GET;
    }

    public Webspider( Site toSpider ){
        this();
        m_site = toSpider;
    }
    
    public Webspider( Site toSpider, int requestMethod ){
        this(toSpider);
        m_requestMethodType = requestMethod;
    }
//------------------------------------------------------------------------------
    private HttpMethod getRequestMethodClass(){
        switch( m_requestMethodType ){
            case WebGlobal.REQUEST_METHOD_GET:
                return (new GetMethod());
            case WebGlobal.REQUEST_METHOD_HEAD:
                return (new HeadMethod());
            case WebGlobal.REQUEST_METHOD_POST:
                return (new PostMethod());
            case WebGlobal.REQUEST_METHOD_DELETE:
                return (new DeleteMethod());
            default:
                System.err.println("Webspider::getRequestMethodClass() : unkown method requestes" 
                                    + " setting to GET method");
                return (new GetMethod());
        }
    }
//------------------------------------------------------------------------------
    private void prepareRequestMethod( URLInfo url , HttpMethod requestMethod, boolean redirect ){

        requestMethod.setPath( url.getPath() );
        if( url.getQuery() != "" )
            requestMethod.setQueryString( url.getQuery() );
        
        //requestMethod.addRequestHeader("Accept", "text/html");
        requestMethod.addRequestHeader("User-Agent", WebGlobal.USER_AGENT);
        
        requestMethod.setFollowRedirects( redirect );
    }
//------------------------------------------------------------------------------    
    protected void setInfoToSite( URLData url ){
        FileURL newFile = null;
        
        switch( url.getURLType() ){
            case WebGlobal.URL_TYPE_FILE:
                if( WebGlobal.DEBUG )
                    System.err.println("Webspider::setInfoToSite() : URL_TYPE_FILE for : " + url.toString() );
                
                try{
                    newFile = new FileURL(url);
                }
                catch( MalformedURLException urlEx ){
                    System.err.println("Webspider::setInfoToSite() : " + urlEx);
                    return;
                }
                m_site.addFileToFolder(url.getFolder(), newFile);
                break;
            case WebGlobal.URL_TYPE_FOLDER:
                if( WebGlobal.DEBUG )
                    System.err.println("Webspider::setInfoToSite() : URL_TYPE_FOLDER for : " + url.toString() );
                
                m_site.addFolder( url.getFolder() );
                break;
            default:
                System.err.println("Webspider::setInfoToSite : CRITICAL : unknown URL TYPE SPECIFIED : " + url.getURLType() );
                System.err.println("aborting!Please report this");
                System.exit(1);
        }
    }
//------------------------------------------------------------------------------
    protected void processFileInfo( HTMLFile fileInfo, URLData url ){
        
        // setting URLData info
        
        // adding links to the file
        url.addLinkList( fileInfo.getURLList() );
        
        processFileURLList( fileInfo, url );
        processEmailList( fileInfo, url );
        processCommentList( fileInfo, url );
        
        // setting the site info.
        setInfoToSite(url);
    }
//------------------------------------------------------------------------------
    private boolean isPotentialHTMLFile( URLInfo url ){
        FileURL fileToCheck = null;
        
        try{
            fileToCheck = new FileURL(url);
        }
        catch( MalformedURLException urlEx ){
            return false;
        }
        
        /*
         *  It's a folder, good chance it links to an html file
         */
        if( fileToCheck.getFile() == "" )
            return true;
        
        for( int i = 0; i < WebGlobal.POTENTIAL_HTML_FILE_EXT.length; i++ ){
            if( fileToCheck.getExtension().equalsIgnoreCase(WebGlobal.POTENTIAL_HTML_FILE_EXT[i]) ){
                if( WebGlobal.DEBUG )
                    System.err.println("Webspider::isPotentialHTMLFile() : found html file for : " + url.getURL() );
                
                return true;
            }
        }
        return false;
    }
//------------------------------------------------------------------------------    
    protected void processFileURLList( HTMLFile fileInfo, URLData url ){
        ArrayList urlList = null;
        //boolean stillGoOn = true;   // for illegalCaracters check
        String strURL = null;
        URLData newURLInfo = null;
        URLInfo urlBaseInfo = null;
        URLInfo wouldBeSameAs = null;

        /*
         * This fix some of the bug I've see when the code tries to fetch
         * java script.
         */
        //String illegalCaracters[] = { "(", ")", "{", "}" };
        
        urlList = fileInfo.getURLList();
        
        try{
            urlBaseInfo = new URLInfo( url.getBaseURL() );
        }
        catch( MalformedURLException urlEx ){
            // This is a serious problem if we get this here
            System.err.println("Webspider::processFileURLList : " + urlEx );
        }
        
        for( int i = 0; i < urlList.size(); i++ ){
            strURL = (String)urlList.get(i);
            
            /*
             * Checking for the presence of javascript() in the url
             *
             * Todo:
             *  manage javascript in URL in URLInfo
             *
             */
            if( strURL.toLowerCase().indexOf("javascript") != -1 ){
                if( WebGlobal.DEBUG )
                    System.err.println("Webspider::processFileURLList() : found javascript in url : " + strURL );
                
                continue;
            }

            /*
             * TODO
             *  - see if this is really needed
             *
            for( int illegalCharIndex = 0; illegalCharIndex < illegalCaracters.length; illegalCharIndex++ ){
                URLInfo urlToCheck = new URLInfo(strURL);
                
                if( urlToCheck.getPath().indexOf(illegalCaracters[illegalCharIndex]) != -1 ){
                    stillGoOn = false;
                    break;
                }
            }
            if( !stillGoOn )
                continue;
                
            */
            
            /*
             *  Checking for creation of the same url as the parent url
             *
             *  Logic:
             *  Check if the combinaison the strURL and url.getBaseURL() would create the same
             *  link referenced to the current url. If it would, we simply skip it.
             *  This goes on for all the strURL that will be added to the parent URL.
             *  This is accomplish by the code using the variable wouldBeSameAs.
             */
             
            if( strURL.startsWith("_top") || strURL.startsWith("#") )       //getting rid of the reference to same doc
                continue;
            else if( strURL.startsWith("http://") || strURL.startsWith("https://") ){   // full URL
                
                /*
                 *  Checking if the url referencing itself
                 */
                if( strURL.compareToIgnoreCase(url.getURL()) == 0 ){
                    if( WebGlobal.DEBUG )
                        System.err.println("Webspider::processFileURLList() : strURL : " + strURL + " same as url : " + url.getURL() );
                    
                    continue;
                }
                try{
                    newURLInfo = new URLData(strURL);
                }
                catch( MalformedURLException urlEx ){
                    System.err.println("WebSpider::processFileURLList() : protocol " + urlEx );
                    continue;
                }
            }
            else if( strURL.lastIndexOf("/") == -1 ){           // file in same folder : Example : ulref = http://www.test.com/index.html  strURL = index2.html
                try{
                    wouldBeSameAs = new URLData( strURL, urlBaseInfo );
                }
                catch( MalformedURLException urlEx ){
                    System.err.println("WebSpider::processFileURLList() : protocol " + urlEx );
                    continue;
                }
                
                if( wouldBeSameAs.sameAs(url) ){
                    if( WebGlobal.DEBUG )
                        System.err.println("WebSpider::processFileURLList() : new url " + strURL + " with main url base " + urlBaseInfo.getURL() + " would be same as " + url.getURL() );
                    
                    continue;
                }
                
                try{
                    newURLInfo = new URLData(strURL, url);
                }
                catch( MalformedURLException urlEx ){
                    System.err.println("WebSpider::processFileURLList()-1 : " + urlEx );
                    continue;
                }                
                if( WebGlobal.DEBUG )
                    System.err.println("WebSpider::processFileURLList-1 : relative url : urlRef : " + url.getURL() + " | strURL = " + strURL + " | newURL : " + newURLInfo.getURL() );
               
            }
            else if( (!strURL.startsWith("/")) && (strURL.lastIndexOf("/") != -1) ){    // folder/file or folder/file/ combo : the folder link is not preceded by a /
                try{
                    wouldBeSameAs = new URLInfo( strURL, urlBaseInfo );
                }
                catch( MalformedURLException urlEx ){
                    System.err.println("WebSpider::processFileURLList()-2 :" + urlEx );
                    continue;
                }
                
                if( wouldBeSameAs.sameAs(url) ){
                    if( WebGlobal.DEBUG )
                        System.err.println("WebSpider::processFileURLList() : new url " + strURL + " with main url base " + urlBaseInfo.getURL() + " would be same as " + url.getURL() );
                    
                    continue;
                }
                
                try{
                    newURLInfo = new URLData(strURL, url);
                }
                catch( MalformedURLException urlEx ){
                    System.err.println("WebSpider::processFileURLList()-2 :" + urlEx );
                    continue;
                }
                
                if( WebGlobal.DEBUG )
                    System.err.println("WebSpider::processFileURLList-2 : relative url : urlRef : " + url.getURL() + " | strURL = " + strURL + " | newURL : " + newURLInfo.getURL() );
               
            }
            else if( strURL.startsWith("/") ){  // Example : urlref = http://www.test.com strURL = /testing123

                try{
                    wouldBeSameAs = new URLData( strURL, urlBaseInfo );
                }
                catch( MalformedURLException urlEx ){
                    System.err.println("WebSpider::processFileURLList()-3 :" + urlEx );
                    continue;
                }
                
                if( wouldBeSameAs.sameAs(url) ){
                    if( WebGlobal.DEBUG )
                        System.err.println("WebSpider::processFileURLList() : new url " + strURL + " with main url base " + urlBaseInfo.getURL() + " would be same as " + url.getURL() );
                    
                    continue;
                }
                
                /*
                 * Here we create the new url with the parent base url since
                 * the link starts at the root.
                 */
                try{
                    newURLInfo = new URLData( strURL, urlBaseInfo );
                }
                catch( MalformedURLException urlEx ){
                    System.err.println("WebSpider::processFileURLList()-3 :" + urlEx );
                    continue;
                }                

                if( WebGlobal.DEBUG )
                    System.err.println("WebSpider::processFileURLList-3 : relative url : urlRef : " + url.getURL() + " | strURL = " + strURL + " | newURL : " + newURLInfo.getURL() );
            
            }
            else{
                try{
                    newURLInfo = new URLData(strURL);
                }
                catch( MalformedURLException urlEx ){
                    System.err.println("WebSpider::processFileURLList() :" + urlEx );
                    continue;
                }                
            }
            
            if( !m_site.isURLAlreadyDone( (URLInfo)newURLInfo) ){
                m_site.addURLDone( (URLInfo)newURLInfo );
                
                /*
                 * Checking if the new url is in the same domain as the other one
                 * if not, we just continue to loop and we don't go fetch it
                 */
                if( url.getHost().compareToIgnoreCase(newURLInfo.getHost()) != 0 ){
                    if( WebGlobal.DEBUG )
                        System.err.println("Webspider::processFileURLList() : " + newURLInfo.getURL() + " not in same domain ");

                    m_site.addOutsideURL( (URLInfo)newURLInfo );
                    continue;
                }
                else if( newURLInfo.getPort() != url.getPort() ){
                    if( WebGlobal.DEBUG )
                        System.err.println("Webspider::processFileURLList() : " + newURLInfo.getURL() + " port : " + newURLInfo.getPort() );
                    
                    m_site.addOutsideURL( (URLInfo)newURLInfo );
                    continue;
                }
                else if( !newURLInfo.getProtocol().equalsIgnoreCase(url.getProtocol()) ){
                    if( WebGlobal.DEBUG )
                        System.err.println("Webspider::processFileURLList() : " + newURLInfo.getURL() + " protocol : " + newURLInfo.getProtocol() );
                    
                    m_site.addOutsideURL( (URLInfo)newURLInfo );
                    continue;
                        
                }
                
                getFileInfo(newURLInfo);
            }
            else{
                if( WebGlobal.DEBUG )
                    System.err.println("Webspider::processFileURLList() : url already done : " + newURLInfo.getURL() );
            }
        }
    }
//------------------------------------------------------------------------------
    protected void processEmailList( HTMLFile fileInfo, URLData url ){
        ArrayList foundEmails = fileInfo.getEmailFound();
        EmailURL newEmail = null;
        
        for( int i = 0; i < foundEmails.size(); i++ ){
            String emailStr = (String)foundEmails.get(i);
            try{
                newEmail = new EmailURL(emailStr);
                // we add it to the site emails
                m_site.addEmail(newEmail);
            
                //we add it to the file
                url.addEmail(newEmail);
            }
            catch( MalformedURLException urlEx ){
                if( WebGlobal.DEBUG )
                    System.err.println("Webspider::processEmailList() : " + urlEx + " on : " + url.getURL() );
            }
        }
    }
//------------------------------------------------------------------------------    
    public void run(){
        
        URLData url = null;
        
        try{
            url = new URLData( m_site.getInfo() );
        }
        catch( MalformedURLException urlEx ){
            System.err.println("WebSpider::processFileURLList()-2 :" + urlEx );
            return;
        }        
        
        //setInfoToSite( url );
        m_site.addURLDone( (URLInfo)url );
        getFileInfo( url );
        
    }
//------------------------------------------------------------------------------
    public void setRequestMethod( int requestMethodType ){
        m_requestMethodType = requestMethodType;
    }
//------------------------------------------------------------------------------
    protected boolean getFileInfo( URLData url ){
        URLCont urlData = null;
        HttpMethod requestMethod = null;
        int requestCode;
        boolean allGood = true;        
          
        if( WebGlobal.DEBUG )
             System.err.println("\nWebspider::getFileInfo() : getting url : " + url.getURL() );
        
        urlData = new URLCont( (URLInfo)url );
        requestMethod = (HttpMethod)new HeadMethod();
        prepareRequestMethod( (URLInfo)url, requestMethod, WebGlobal.FOLLOW_INSIDE_REDIRECT );

        // get rid of expired cookies
        urlData.purgeCookies();
        
        try{
            urlData.open();
        }
        catch( IOException openIoEx ){
            System.err.println("Webspider::getFileInfo() : " + openIoEx);
            return false;
        }
            
        try{
            requestCode = requestMethod.execute(urlData.getState(), urlData);
            
            // Checking return code
            switch( requestCode ){
                // ERRORS
                case HttpStatus.SC_BAD_REQUEST:             //  400 Bad Request (HTTP/1.1 - RFC 2616)
                case HttpStatus.SC_NOT_FOUND:               //  404 Not Found (HTTP/1.0 - RFC 1945)

                    /*
                     *  If the url is a query, we try to fetch the data anyways
                     *  since we don't set allGood to false, it will we send to getFileData()
                     */
                    if( url.getQuery() == "" )
                        allGood = false;

                    break;
                case HttpStatus.SC_GONE:                    //  410 Gone (HTTP/1.1 - RFC 2616)
                    allGood = false;
                    break;
                case HttpStatus.SC_NOT_ACCEPTABLE:  //  406 Not Acceptable (HTTP/1.1 - RFC 2616)
                    if( !isPotentialHTMLFile( (URLInfo)url ) )
                        allGood = false;
                    else{
                        if( WebGlobal.DEBUG )
                            System.err.println("Webspider::getFileInfo() : potential html for file : " + url.getFile() + " for url : " + url.getURL() );
                    }
                    break;
                    
                // Interesting errors
                case HttpStatus.SC_INTERNAL_SERVER_ERROR:   //  500 Server Error (HTTP/1.0 - RFC 1945)
                case HttpStatus.SC_SERVICE_UNAVAILABLE:    //  503 Service Unavailable (HTTP/1.0 - RFC 1945)
                    System.err.println("Webspider::getFileInfo() : got Server Error " + requestCode + " on : " + url.getURL() );
                    allGood = false;
                    break;
                    
                // section of error codes that indicate that the page is present but we can't see them
                case HttpStatus.SC_UNAUTHORIZED:            //  401 Unauthorized (HTTP/1.0 - RFC 1945)
                    /*
                     *  We add the url here since we won't be able to fetch it later
                     */
                    setInfoToSite(url);
                    allGood = false;
                    break;
                    
                // Section of error codes that I want to see if I get them
                case HttpStatus.SC_PAYMENT_REQUIRED:        //  402 Payment Required (HTTP/1.1 - RFC 2616) : This code is reserved for future use.           
                case HttpStatus.SC_FORBIDDEN:               //  403 Forbidden (HTTP/1.0 - RFC 1945)
                case HttpStatus.SC_METHOD_NOT_ALLOWED:      //  405 Method Not Allowed (HTTP/1.1 - RFC 2616)
                case HttpStatus.SC_REQUEST_TIMEOUT:         //  408 Request Timeout (HTTP/1.1 - RFC 2616)
                case HttpStatus.SC_CONFLICT:                //  409 Conflict (HTTP/1.1 - RFC 2616)
                case HttpStatus.SC_LENGTH_REQUIRED:         //  411 Length Required (HTTP/1.1 - RFC 2616)
                case HttpStatus.SC_PRECONDITION_FAILED:     //  412 Precondition Failed (HTTP/1.1 - RFC 2616)
                case HttpStatus.SC_REQUEST_TOO_LONG:        //  413 Request Entity Too Large (HTTP/1.1 - RFC 2616)
                case HttpStatus.SC_REQUEST_URI_TOO_LONG:    //  414 Request-URI Too Long (HTTP/1.1 - RFC 2616)
                case HttpStatus.SC_UNSUPPORTED_MEDIA_TYPE:  //  415 Unsupported Media Type (HTTP/1.1 - RFC 2616)
                case HttpStatus.SC_REQUESTED_RANGE_NOT_SATISFIABLE:     //  416 Requested Range Not Satisfiable (HTTP/1.1 - RFC 2616)
                case HttpStatus.SC_EXPECTATION_FAILED:      //  417 Expectation Failed (HTTP/1.1 - RFC 2616)
                case HttpStatus.SC_INSUFFICIENT_SPACE_ON_RESOURCE:      //  419 Insufficient Space on Resource (WebDAV - draft-ietf-webdav-protocol-05?) or 419 Proxy Reauthentication Required (HTTP/1.1 drafts?)
                case HttpStatus.SC_METHOD_FAILURE:          //  420 Method Failure (WebDAV - draft-ietf-webdav-protocol-05?)
                case HttpStatus.SC_UNPROCESSABLE_ENTITY:    //  422 Unprocessable Entity (WebDAV - RFC 2518)
                case HttpStatus.SC_LOCKED:                  //  423 Locked (WebDAV - RFC 2518)
                case HttpStatus.SC_FAILED_DEPENDENCY:       //  424 Failed Dependency (WebDAV - RFC 2518)
                case HttpStatus.SC_NOT_IMPLEMENTED:         //  501 Not Implemented (HTTP/1.0 - RFC 1945)
                case HttpStatus.SC_HTTP_VERSION_NOT_SUPPORTED:      //  505 HTTP Version Not Supported (HTTP/1.1 - RFC 2616)
                case HttpStatus.SC_INSUFFICIENT_STORAGE:    //  507 Insufficient Storage (WebDAV - RFC 2518)
                    System.err.println("Webspider::getFileInfo() : got un usual error " + requestCode + " on : " + url.getURL() );
                    allGood = false;
                    break;

                // section indicating proxy error
                case HttpStatus.SC_PROXY_AUTHENTICATION_REQUIRED:   // 407 Proxy Authentication Required (HTTP/1.1 - RFC 2616)
                case HttpStatus.SC_BAD_GATEWAY:             //  502 Bad Gateway (HTTP/1.0 - RFC 1945)
                case HttpStatus.SC_GATEWAY_TIMEOUT:         //  504 Gateway Timeout (HTTP/1.1 - RFC 2616)
                    System.err.println("Webspider::getFileInfo() : got proxy error " + requestCode + " on : " + url.getURL() );
                    allGood = false;
                    break;
            }
        }
        catch( HttpException httpEx ){
            System.err.println("Webspider::getFileInfo() : " + httpEx);
            allGood = false;
        }
        catch( IOException ioExRequest ){
            System.err.println("Webspider::getFile() : " + ioExRequest.toString());
            allGood = false;
        }
                
        //set the request Information
        url.setResponseInfo( requestMethod.getResponseHeaders() );

        if( url.getContentTypeInfo().toLowerCase().indexOf("text/html") == -1 ){
        //if( !(url.getContentTypeInfo().compareToIgnoreCase("text/html") == 0) ){
            if( WebGlobal.DEBUG )
                System.err.println("WebSpider::getFileInfo() : Content-Type : " + url.getContentTypeInfo() + " on url : " + url.getURL() );
                    
                /*
                 *  If the url is a query, we try to fetch the data anyways
                 *  since we don't set allGood to false, it will we send to getFileData()
                 */
                if( (url.getQuery() == "") || (url.getQuery() == null) )
                    allGood = false;
                
            /*
             *  We add the url here since it will never be added later in processFileInfo()
             */
            setInfoToSite(url);
        }

        try{
            urlData.close();
        }
        catch( IOException closeIoEx ){
            System.err.println("WebSpider::getFileInfo() : " + closeIoEx );
        }
        
        // if everything is good, we pass the info to the handler
        if( allGood )
            getFileData(url);
        
        return allGood;
    }
//------------------------------------------------------------------------------    
    protected boolean getFileData( URLData url ){

        HTMLFile fileInfo = null;
        URLCont urlData = null;
        HTMLEditorKit.Parser parser = null;
        HTMLEditorKit.ParserCallback callback = null;
        HttpMethod requestMethod = null;
        boolean allGood = true;
        int requestCode;
        URLInfo tmpURL = null;
        String urlLocation = null;
        
        fileInfo = new HTMLFile();
        
        if( WebGlobal.DEBUG )
            System.err.println("WebSpider::getFileData() : getting url : " + url.getURL());
        
        try{
            ParserGetter kit = new ParserGetter();
            parser = kit.getParser();
            callback = fileInfo;
            
            try{
                urlData = new URLCont( (URLInfo)url );
                requestMethod = getRequestMethodClass();
                prepareRequestMethod( (URLInfo)url, requestMethod, WebGlobal.FOLLOW_INSIDE_REDIRECT );

                requestMethod.addRequestHeader("Accept", "text/html");
                //urlData.setMethod(requestMethod);
                
                // 1.5 Mb should be enought
                ((GetMethod)requestMethod).setMaxBodySize(1572864);
                
                // get rid of expired cookies
                urlData.purgeCookies();
                
                try{
                    urlData.open();
                }
                catch( IOException openIoEx ){
                    System.err.println("Webspider::getFileData() : " + openIoEx);
                    return false;
                }                
                try{
                    requestCode = requestMethod.execute(urlData.getState(), urlData);

                    // Checking return code
                    switch( requestCode ){
                        // section of error codes that indicate that the page is present but we can't see them
                        case HttpStatus.SC_UNAUTHORIZED:            //  401 Unauthorized (HTTP/1.0 - RFC 1945)
                            /*
                             *  We add the url here since we won't be able to fetch it later
                             */
                            setInfoToSite(url);
                            allGood = false;
                            break;                        
                        // ERRORS
                        case HttpStatus.SC_BAD_REQUEST:             //  400 Bad Request (HTTP/1.1 - RFC 2616)
                        case HttpStatus.SC_NOT_FOUND:               //  404 Not Found (HTTP/1.0 - RFC 1945)
                        case HttpStatus.SC_GONE:                    //  410 Gone (HTTP/1.1 - RFC 2616)
                        case HttpStatus.SC_NOT_ACCEPTABLE:  //  406 Not Acceptable (HTTP/1.1 - RFC 2616)
                            
                        // Interesting errors
                        case HttpStatus.SC_INTERNAL_SERVER_ERROR:   //  500 Server Error (HTTP/1.0 - RFC 1945)
                        case HttpStatus.SC_SERVICE_UNAVAILABLE:    //  503 Service Unavailable (HTTP/1.0 - RFC 1945)

                        // Section of error codes that I want to see if I get them
                        case HttpStatus.SC_PAYMENT_REQUIRED:        //  402 Payment Required (HTTP/1.1 - RFC 2616) : This code is reserved for future use.           
                        case HttpStatus.SC_FORBIDDEN:               //  403 Forbidden (HTTP/1.0 - RFC 1945)
                        case HttpStatus.SC_METHOD_NOT_ALLOWED:      //  405 Method Not Allowed (HTTP/1.1 - RFC 2616)
                        case HttpStatus.SC_REQUEST_TIMEOUT:         //  408 Request Timeout (HTTP/1.1 - RFC 2616)
                        case HttpStatus.SC_CONFLICT:                //  409 Conflict (HTTP/1.1 - RFC 2616)
                        case HttpStatus.SC_LENGTH_REQUIRED:         //  411 Length Required (HTTP/1.1 - RFC 2616)
                        case HttpStatus.SC_PRECONDITION_FAILED:     //  412 Precondition Failed (HTTP/1.1 - RFC 2616)
                        case HttpStatus.SC_REQUEST_TOO_LONG:        //  413 Request Entity Too Large (HTTP/1.1 - RFC 2616)
                        case HttpStatus.SC_REQUEST_URI_TOO_LONG:    //  414 Request-URI Too Long (HTTP/1.1 - RFC 2616)
                        case HttpStatus.SC_UNSUPPORTED_MEDIA_TYPE:  //  415 Unsupported Media Type (HTTP/1.1 - RFC 2616)
                        case HttpStatus.SC_REQUESTED_RANGE_NOT_SATISFIABLE:     //  416 Requested Range Not Satisfiable (HTTP/1.1 - RFC 2616)
                        case HttpStatus.SC_EXPECTATION_FAILED:      //  417 Expectation Failed (HTTP/1.1 - RFC 2616)
                        case HttpStatus.SC_INSUFFICIENT_SPACE_ON_RESOURCE:      //  419 Insufficient Space on Resource (WebDAV - draft-ietf-webdav-protocol-05?) or 419 Proxy Reauthentication Required (HTTP/1.1 drafts?)
                        case HttpStatus.SC_METHOD_FAILURE:          //  420 Method Failure (WebDAV - draft-ietf-webdav-protocol-05?)
                        case HttpStatus.SC_UNPROCESSABLE_ENTITY:    //  422 Unprocessable Entity (WebDAV - RFC 2518)
                        case HttpStatus.SC_LOCKED:                  //  423 Locked (WebDAV - RFC 2518)
                        case HttpStatus.SC_FAILED_DEPENDENCY:       //  424 Failed Dependency (WebDAV - RFC 2518)
                        case HttpStatus.SC_NOT_IMPLEMENTED:         //  501 Not Implemented (HTTP/1.0 - RFC 1945)
                        case HttpStatus.SC_HTTP_VERSION_NOT_SUPPORTED:      //  505 HTTP Version Not Supported (HTTP/1.1 - RFC 2616)
                        case HttpStatus.SC_INSUFFICIENT_STORAGE:    //  507 Insufficient Storage (WebDAV - RFC 2518)

                        // section indicating proxy error
                        case HttpStatus.SC_PROXY_AUTHENTICATION_REQUIRED:   // 407 Proxy Authentication Required (HTTP/1.1 - RFC 2616)
                        case HttpStatus.SC_BAD_GATEWAY:             //  502 Bad Gateway (HTTP/1.0 - RFC 1945)
                        case HttpStatus.SC_GATEWAY_TIMEOUT:         //  504 Gateway Timeout (HTTP/1.1 - RFC 2616)
                            allGood = false;
                            break;
                    }
                }
                catch( HttpException httpEx ){
                    System.err.println("Webspider::getFileData() : " + httpEx);
                    allGood = false;
                }
                catch( IOException ioExRequest ){
                    System.err.println("Webspider::getFileData() : " + ioExRequest.toString());
                    allGood = false;
                }
                
                //set the request Information
                url.setResponseInfo( requestMethod.getResponseHeaders() );

                if( url.getContentTypeInfo().toLowerCase().indexOf("text/html") == -1 ){
                        System.err.println("WebSpider::getFileData() : Content-Type : " + url.getContentTypeInfo() + " on url : " + url.getURL() );
                    
                    allGood = false;
                }
                
                /*
                 * TODO :
                 *  - Add contentLocationInfo support
                 *
                 *Logic:
                 *  we check if the location header as data.
                 *  If there is data, we check if the location is the same
                 *  as the current url. We also check if this location is already
                 *  done. If both are false, we create a new URLData and send it to
                 *  getFile
                
                if( (urlLocation = url.getContentLocationInfo()) != null ){
                    tmpURL = new URLInfo(urlLocation);
                    
                    if( !tmpURL.sameAs((URLInfo)url) ){
                        if( !m_site.isURLAlreadyDone(tmpURL.toString()) ){
                            URLData newURL = new URLData(tmpURL);
                            setInfoToSite(newURL);
                        }
                    }
                }
                */
                // if everything is good, we pass the info to the handler
                if( allGood ){
                    InputStream in = requestMethod.getResponseBodyAsStream();
                    InputStreamReader r = new InputStreamReader(in);                                
                    parser.parse(r, callback,true);
                }
            }
            catch( IOException ioEx ){
                System.err.println("WebSpider::getFileData() : IOException : " + ioEx );
                allGood = false;
            }
        }
        catch( Exception e ){
            System.err.println("WebSpider::getFileData() : exception : " + e );
            e.printStackTrace();
            allGood = false;     
        }
        finally{
            try{
                urlData.close();
            }
            catch( IOException closeIoEx ){
                System.err.println("WebSpider::getFileData() : " + closeIoEx );
            }
        }
            
        // page 251 :Java Network Programming
        try{
            callback.flush();
        }
        catch( BadLocationException badLocE ){
            System.err.println("WebSpider::getFileData() : " + badLocE );
        }
        
        //setInfoToSite(url);
        processFileInfo( fileInfo, url );
        return allGood;
    }
    
//------------------------------------------------------------------------------
    private void processCommentList( HTMLFile fileInfo, URLData url ){
        ArrayList comments = fileInfo.getComments();
        
        if( comments.size() > 0 )
            url.addCommentList( comments );
    }
//------------------------------------------------------------------------------    
    private void DEBUG_showUrlsDone(){
        System.out.println("URLS DONE:");
        ArrayList doneList = m_site.getUrlsDone();
        for( int i = 0; i < doneList.size(); i++ )
            System.out.println("url["+i+"] : " + (String)doneList.get(i));
    }    
//------------------------------------------------------------------------------
    /*
    private void DEBUG_dumpData(){
        Hashtable folderHash = m_site.getFolderHashTable();
        Enumeration folderKeys = folderHash.keys();
        int folderIndex = 0;
        
        System.out.println("\nSITE DATA");
        while( folderKeys.hasMoreElements() ){
            String key = (String)folderKeys.nextElement();
            ArrayList fileList = (ArrayList)folderHash.get(key);
            System.out.println("Folder[" + folderIndex + "] : " + key );
            for( int i = 0; i < fileList.size(); i++ ){
                FileURL file = (FileURL)fileList.get(i);
                System.out.println("\tFile[" + i + "] : " + file.getFileName());
                System.out.println("\t\tExtension : " + file.getExtension());
                System.out.println("\t\tServer : " + file.getServerInfo());
                System.out.println("\t\tContent-Length : " + file.getContentLengthInfoAsString());
                System.out.println("\t\tContent-Type : " + file.getContentTypeInfo());
                System.out.println("\t\tCookie : " + file.getCookieInfo());
                System.out.println("\t\tLast Modified : " + file.getLastModifiedInfo());
                System.out.println("\t\tLocation : " + file.getContentLocationInfo());
            }
            folderIndex++;
        }
    }
     */
//------------------------------------------------------------------------------    
    /*
    public static void main( String args[] ){
        //URLInfo u = new URLInfo("http://jetson.wox.org");
        //URLInfo u = new URLInfo("http://bukakke");
        URLInfo u = new URLInfo("http://www.severus.org/sacha");
        Site s = new Site(u);
        Webspider t = new Webspider(s);
        
        t.run();
        
        //t.DEBUG_showUrlsDone();
        //t.DEBUG_dumpData();
        
        System.exit(0);
    }
     */
}
