/*
** Copyright (C) 2001,2002 Sacha Faust <sacha@severus.org>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
/*
 * WebGlobal.java
 *
 * Created on October 31, 2001, 9:50 PM
 */

package faust.sacha.web.util;


/**
 *
 * @author Sacha Faust
 * @version 1.0
 */

//import java.security.*;
import java.util.*;
import java.io.*;

public final class WebGlobal {
    public static final int HTTP_DEFAULT_PORT               = 80;
    public static final int HTTPS_DEFAULT_PORT              = 443;
    public static final String PROTOCOL_DEFAULT             = "http://";
    
    public static final String PROTOCOL_LIST[] = {
                                                    "http://",
                                                    "https://",
                                                    "ftp://",
                                                    "telnet://",
                                                    "rmi://",
                                                    "ldap://",
                                                    "mailto:",
                                                 };
                                                    
    
    public static final int REQUEST_METHOD_GET              = 1;
    public static final int REQUEST_METHOD_HEAD             = 2;
    public static final int REQUEST_METHOD_POST             = 3;
    public static final int REQUEST_METHOD_DELETE           = 4;
    
    public static int DEFAULT_PORT                          = HTTP_DEFAULT_PORT;
    public static int REQUEST_METHOD                        = REQUEST_METHOD_GET;
    
    public static final int URL_TYPE_FILE                   = 1;
    public static final int URL_TYPE_FOLDER                 = 2;
    
    public static boolean DEBUG                             = false;
    public static String USER_AGENT                         = "Riding the Snake v1.1";
    
    public static String PROXY_HOST                         = null;
    public static int PROXY_PORT                            = 80;
    //public static String PROXY_USERNAME                     = null;
    //public static String PROXY_PASSWORD                     = null;
    
    
    public static boolean FOLLOW_INSIDE_REDIRECT            = false;
    public static boolean FOLLOW_OUTSIDE_REDIRECT           = false;
    
    public static boolean USE_PROXY                         = false;
    
    public static String POTENTIAL_HTML_FILE_EXT[]    = {
                                                            "htm",
                                                            "html",
                                                            "shtml",
                                                            "stm",      //  jeff site
                                                            "pl",       //  perl
                                                            "cfm",      //  Cold Fusion
                                                            "php",      //  PHP
                                                            "php3",     //  PHP
                                                            "mv",       //  Miva Merchand
                                                            "cgi",
                                                            "asp",      //  Active Server Pages
                                                            "css"   };
                                                            

    
    private static String PROPERTY_FILE_NAME = "webglobal.properties";
    
////////////////////////////////////////////////////////////////////////////////   
    
    public WebGlobal() {
    }
//------------------------------------------------------------------------------
    public static void init(){
        Properties props = null;
        Enumeration propKeys = null;
        
        if( (props = loadPropertiesFile()) == null ){
            props = new Properties();
            fillProperties(props);
            writePropertyFile();
            
        }
        else{
            USER_AGENT = props.getProperty("useragent", USER_AGENT);
            resetHTMLExtListFromProperties( props.getProperty("potential_html_file_ext", null) );

            if(  (PROXY_HOST = props.getProperty("proxy_host")).equals("") )
                PROXY_HOST = null;
            
            
            try{
                // ether I'm very tired but casting int to boolean shouldn't be this complicated ...
                DEBUG = Integer.decode( props.getProperty("debug", String.valueOf( (DEBUG?1:0)) ) ).intValue() == 1 ? true : false;
                FOLLOW_INSIDE_REDIRECT = Integer.decode( props.getProperty("follow_inside_redirect", String.valueOf(FOLLOW_INSIDE_REDIRECT)) ).intValue() == 1 ? true : false;
                FOLLOW_OUTSIDE_REDIRECT = Integer.decode( props.getProperty("follow_outside_redirect", String.valueOf(FOLLOW_OUTSIDE_REDIRECT)) ).intValue() == 1 ? true : false;
                DEFAULT_PORT = Integer.decode( props.getProperty("default_port", String.valueOf(DEFAULT_PORT)) ).intValue();
                PROXY_PORT = Integer.decode( props.getProperty("proxy_port", String.valueOf(PROXY_PORT)) ).intValue();
            }
            catch( NumberFormatException numEx ){
                System.err.println("Unable to parse property file : " + numEx + " . Will be using default!");
            }
        }
    }
//------------------------------------------------------------------------------
    private static void fillProperties( Properties props ){
            props.put( (Object)"useragent", (Object)USER_AGENT );
            props.put( (Object)"debug", (Object)String.valueOf(DEBUG ? 1 : 0) );
            props.put( (Object)"follow_inside_redirect", (Object)String.valueOf(FOLLOW_INSIDE_REDIRECT ? 1 : 0 ) );
            props.put( (Object)"follow_outside_redirect", (Object)String.valueOf(FOLLOW_OUTSIDE_REDIRECT ? 1 : 0) );
            props.put( (Object)"default_port", (Object)String.valueOf(DEFAULT_PORT) );
            props.put( (Object)"potential_html_file_ext", (Object)getFileExtList() );
            props.put( (Object)"proxy_host", (Object)PROXY_HOST );
            props.put( (Object)"proxy_port", (Object)String.valueOf(PROXY_PORT) );
    }
//------------------------------------------------------------------------------   
    private static Properties loadPropertiesFile(){
        Properties props = null;
        File propFile = null;
        FileInputStream fileOut = null;
        
        try{
            propFile = new File(PROPERTY_FILE_NAME);
        }
        catch( NullPointerException nullEx ){
            return null;
        }
        
        if( !propFile.exists() )
            return null;
        
        try{
            propFile.createNewFile();
            fileOut = new FileInputStream(propFile);
            props = new Properties();
            props.load(fileOut);
        }
        catch( IOException ioEx ){
            return null;
        }
        
        try{
            fileOut.close();
        }
        catch( IOException ioEx ){
            System.err.println("WebGlobal::loadPropertyFile() : " + ioEx);
        }
        
        return props;
    }
//------------------------------------------------------------------------------
    private static void writePropertyFile(){
        FileOutputStream propFile = null;
        Properties props = new Properties();
        
        try{
            propFile = new FileOutputStream(PROPERTY_FILE_NAME);
        }
        catch( IOException ioEx ){
            System.err.println("WebGlobal::writePropertyFile() : " + ioEx);
            return;
        }
                
        try{
            fillProperties(props);
            props.store( (OutputStream)propFile, "Properties for controling the spider code" );
            propFile.close();
        }
        catch( IOException ioEx ){
            System.err.println("WebGlobal::writePropertyFile() : unable to store properties : " + ioEx);
        }
        catch( ClassCastException  castEx ){
            System.err.println("WebGlobal::writePropertyFile() : cast problem. Please repor this !: " + castEx);
        }
    }
//------------------------------------------------------------------------------    
    private static String getFileExtList(){
        StringBuffer extString = new StringBuffer();

        for( int i = 0; i < POTENTIAL_HTML_FILE_EXT.length; i++ )
            extString.append(POTENTIAL_HTML_FILE_EXT[i] + ";");
        
        return extString.toString();

    }
//------------------------------------------------------------------------------
    private static void resetHTMLExtListFromProperties( String propList ){
        StringTokenizer strToken = null;
        int nbExt = 0;
        String extName = null;
        
        if( propList == null )
            return;
        
        strToken = new StringTokenizer(propList, ";");
        
        if( (nbExt = strToken.countTokens()) == 0 ){
            System.err.println("Unable to parser potential_html_file_ext property. Using defaults!");
            return;
        }
        
        //  reset the string elements
        for( int i = 0; i < POTENTIAL_HTML_FILE_EXT.length; i++ )
            POTENTIAL_HTML_FILE_EXT[i] = null;
        
        //  reset the string array
        POTENTIAL_HTML_FILE_EXT = null;
        POTENTIAL_HTML_FILE_EXT = new String[nbExt];
        
        for( int j = 0; strToken.hasMoreTokens(); j++ ){
            extName = strToken.nextToken();
            if( !extName.equals("") )
                POTENTIAL_HTML_FILE_EXT[j] = new String(extName);
        }
    }
//------------------------------------------------------------------------------
    public static void setDebug( boolean mode ){
        DEBUG = mode;
    }
//------------------------------------------------------------------------------
    public static void setUserAgent( String userAgent ){
        USER_AGENT = userAgent;
    }
//------------------------------------------------------------------------------
    public static void enableSSLDynamically(){
        /*
         * Buggy .jar
         */
        /*
        Security.addProvider(new com.sun.net.ssl.internal.ssl.Provider());

        System.setProperty("java.protocol.handler.pkgs",
                     "com.sun.net.ssl.internal.www.protocol");        
         */
    }
//------------------------------------------------------------------------------
    public static void followInsideRedirect( boolean redirect ){
        FOLLOW_INSIDE_REDIRECT = redirect;
    }
//------------------------------------------------------------------------------
    public static void followOutsideRedirect( boolean redirect ){
        FOLLOW_OUTSIDE_REDIRECT = redirect;
    }
//------------------------------------------------------------------------------
    public static void setRequestMethod( int requestMethod ){
        REQUEST_METHOD = requestMethod;
    }
//------------------------------------------------------------------------------
    public static void DEBUG_DUMP_OPTIONS(){

        System.out.println("DEBUG : " + DEBUG );
        System.out.println("USER_AGENT : " + USER_AGENT );
        System.out.println("FOLLOW_INSIDE_REDIRECT : " + FOLLOW_INSIDE_REDIRECT );
        System.out.println("FOLLOW_OUTSIDE_REDIRECT : " + FOLLOW_OUTSIDE_REDIRECT );
        System.out.println("PROXY_HOST : " + PROXY_HOST );
        System.out.println("PROXy_PORT : " + PROXY_PORT );
        
        System.out.println("HTML_EXT :");
        for( int i = 0; i < POTENTIAL_HTML_FILE_EXT.length; i++ )
            System.out.println("\t" + POTENTIAL_HTML_FILE_EXT[i]);
        
    }
//------------------------------------------------------------------------------    
    public static void main( String args[] ){
        WebGlobal.init();
        WebGlobal.DEBUG_DUMP_OPTIONS();
    }
}
